from flask import Flask, render_template, request, jsonify
from bokeh.plotting import figure, show
from bokeh.embed import components
from bokeh.resources import CDN
from bokeh.layouts import row
import xyzservices.providers as xyz
import backend
import numpy as np
import pandas as pd

app = Flask(__name__)

#get initial values to display on homescreen
conn, cursor = backend.initialize_conn()
latest_co2, latest_pm25, latest_temp, latest_humidity, latest_lat, latest_long = backend.get_latest(cursor)
backend.close_conn(conn)


@app.route('/', methods=['GET', 'POST'])
def index():

    #get latest values for homescreen
    global latest_temp, latest_humidity, latest_pm25, latest_co2, latest_lat, latest_long
        
    #plot latest location for homescreen
    long_merc, lat_merc = wgs84_to_web_mercator(latest_lat, latest_long)
    map_plot = figure(x_range=(-8425584.61, -8425784.61), y_range=(5678025.27, 5698025.27),
                           x_axis_type="mercator", y_axis_type="mercator")
    map_plot.add_tile(xyz.OpenStreetMap.Mapnik)
    map_plot.circle(x=long_merc,y=lat_merc, size=12, fill_color="red", fill_alpha=0.8)
    map_script, map_div = components(map_plot)
    

    if request.method == 'POST':    #homescreen

        cdn_js = CDN.js_files[0]

        if request.form.get('history') == 'Pollution History':

            conn, cursor = backend.initialize_conn()
            x,y1, y2, y3, y4 = backend.get_variation(cursor)    #get variation values for the latest day from db
            backend.close_conn(conn)

            #convert time from string to time format for plotting
            x_formatted = pd.to_datetime(pd.Series(x), format='%H:%M:%S').dt.time

            #y1 - co2, y2 - PM2.5,  y3 - temp, y4 - humidity
            p1 = figure(x_axis_label="time",
                       y_axis_label="variation",
                       x_axis_type='datetime',
                       title="Variation")
            p2 = figure(x_axis_label="time",
                       y_axis_label="variation",
                       x_axis_type='datetime', 
                       title="CO2 variation", 
                       y_range=(500, 20000))
            
            p1.line(x_formatted, y2, line_width=2,
                   legend_label ="PM2.5 concentration", color="black")
            p1.line(x_formatted, y3, line_width=2,
                   legend_label ="Temperature", color="red")
            p1.line(x_formatted, y4, line_width=2,
                   legend_label ="Humidity", color="blue")

            p2.line(x_formatted, y1, line_width=2,
                    legend_label ="CO2 concentration", color="green")
            

            script_p1, div_p1 = components(p1)  #PM25, temp, humidity plot
            script_p2, div_p2 = components(p2)  #CO2 plot

            
            return render_template("plot.html",
                                   script1=script_p1,
                                   div1=div_p1,
                                   script2=script_p2,
                                   div2=div_p2,
                                   cdn_js=cdn_js)



        if request.form.get('about') == 'About Us':
            return render_template("about_us.html")


        if request.form.get('map') == 'Map':
            conn, cursor = backend.initialize_conn()
            lat_points, long_points, co2_conc = backend.get_heatmap_points(cursor)
            backend.close_conn(conn)

            #find the area on the map to be plotted
            if len(lat_points)>0:
                lat_avg = sum(lat_points)/len(lat_points)
                long_avg = sum(long_points)/len(long_points)
                long_avg_merc, lat_avg_merc = wgs84_to_web_mercator(lat_avg, long_avg)
                x1 = long_avg_merc-700.693851693
                x2 = long_avg_merc+700.693851693
                y1 = lat_avg_merc-1000.14474785
                y2 = lat_avg_merc+1000.14474785
            else:
                x1, x2, y1, y2 = -8425584.61, -8425784.61, 5678025.27, 5698025.27   #plot general ottawa area


            heatmap_plot = figure(x_range=(x1,x2), y_range=(y1,y2),
                           x_axis_type="mercator", y_axis_type="mercator")
            heatmap_plot.add_tile(xyz.OpenStreetMap.Mapnik)

            #create dict with x,y,colour to act as a data source for plotting
            data_src = {'x':[],'y':[],'color':[]}
            for i in range(0,len(lat_points)):
                x_pt, y_pt = wgs84_to_web_mercator(lat_points[i],long_points[i])
                color = "green"
                co2_level = float(co2_conc[i])
                if co2_level >= 1000 and co2_level <= 2000:
                    color = "darkorange"
                elif co2_level > 2000 and co2_level <= 5000:
                    color = "orange"
                elif co2_level > 5000 and co2_level <= 40000:
                    color = "orangered"
                elif co2_level > 40000:
                    color = "red"
                data_src['x'].append(x_pt)
                data_src['y'].append(y_pt)
                data_src['color'].append(color)
            
            heatmap_plot.circle(x='x',y='y', size=12, fill_color='color', fill_alpha=0.8, source = data_src)

            
            heatmap_script, heatmap_div = components(heatmap_plot)
            return render_template("map.html",
                                   heatmap_script=heatmap_script,
                                   heatmap_div=heatmap_div,
                                   cdn_js=cdn_js
                                   )


        
    elif request.method == 'GET':
        return render_template("main.html",
                            temp=latest_temp,
                            humidity=latest_humidity,
                            CO2=latest_co2,
                            PM25=latest_pm25,
                            map_script=map_script,
                            map_div=map_div)
    
    return render_template("main.html",
                            temp=latest_temp,
                            humidity=latest_humidity,
                            CO2=latest_co2,
                            PM25=latest_pm25,
                            map_script=map_script,
                            map_div=map_div)




#Save data sent by Arduino
@app.route('/save_data', methods=['POST'])
def save_data():
    try:
        data = request.get_json()
        conn, cursor = backend.initialize_conn()
        backend.insert_data(conn, cursor, data)
        backend.close_conn(conn)
        
        #Update the latest values to be displayed on homescreen
        update_latest(data)
        return "Save Success!"
    except Exception as e:
        return jsonify({"error":str(e)}), 500



#Function to update the latest values to be displayed on homescreen
def update_latest(data):

    global latest_co2, latest_pm25, latest_temp, latest_humidity, latest_lat, latest_long
    
    latest_co2=data['co2']
    latest_temp=data['temperature']
    latest_humidity=data['humidity']
    latest_pm25=data['pm25']
    latest_lat = data['latitude']
    latest_long=data['longitude']


#Function to convert latitude and longitude to web mercator coordinate format
def wgs84_to_web_mercator(latitude, longitude):
      k = 6378137
      x = float(longitude) * (k * np.pi/180.0)
      y = np.log(np.tan((90 + float(latitude)) * np.pi/360.0)) * k
      return x,y
